// import all of boostrap
@import "../libs/bootstrap/scss/bootstrap";

// import fontawesome
$fa-font-path: "../libs/@fortawesome/fontawesome-free/webfonts";
@import "../libs/@fortawesome/fontawesome-free/scss/fontawesome";
@import "../libs/@fortawesome/fontawesome-free/scss/solid";
@import "../libs/@fortawesome/fontawesome-free/scss/brands";
