# Generated by Django 2.2.3 on 2019-07-20 23:31

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="ArchivedSong",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("url", models.CharField(max_length=200, unique=True)),
                ("artist", models.CharField(max_length=1000)),
                ("title", models.CharField(max_length=1000)),
                ("counter", models.IntegerField()),
            ],
        ),
        migrations.CreateModel(
            name="Counter",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("value", models.IntegerField()),
            ],
        ),
        migrations.CreateModel(
            name="CurrentSong",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("queue_key", models.IntegerField()),
                ("votes", models.IntegerField()),
                ("url", models.CharField(max_length=200)),
                ("artist", models.CharField(max_length=1000)),
                ("title", models.CharField(max_length=1000)),
                ("duration", models.CharField(max_length=20)),
                ("location", models.CharField(max_length=1000)),
                ("created", models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name="Pad",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("version", models.IntegerField(default=0)),
                ("content", models.CharField(max_length=100000)),
            ],
        ),
        migrations.CreateModel(
            name="QueuedSong",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("index", models.IntegerField()),
                ("votes", models.IntegerField(default=0)),
                ("url", models.CharField(max_length=200)),
                ("artist", models.CharField(max_length=1000)),
                ("title", models.CharField(max_length=1000)),
                ("duration", models.CharField(max_length=20)),
            ],
            options={"ordering": ["index"],},
        ),
        migrations.CreateModel(
            name="Setting",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("key", models.CharField(max_length=200, unique=True)),
                ("value", models.CharField(max_length=200)),
            ],
        ),
        migrations.CreateModel(
            name="Tag",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("text", models.CharField(max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name="RequestLog",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created", models.DateTimeField(auto_now_add=True)),
                ("address", models.CharField(max_length=50)),
                (
                    "song",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="core.ArchivedSong",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="PlayLog",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created", models.DateTimeField(auto_now_add=True)),
                ("votes", models.IntegerField(null=True)),
                (
                    "song",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="core.ArchivedSong",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="ArchivedQuery",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("query", models.CharField(max_length=1000)),
                (
                    "song",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="queries",
                        to="core.ArchivedSong",
                    ),
                ),
            ],
        ),
    ]
