# Generated by Django 2.2.13 on 2020-07-26 20:12

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0008_auto_20200514_1213"),
    ]

    operations = [
        migrations.DeleteModel(name="Pad",),
    ]
